/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.xml.datatype.XMLGregorianCalendar;

import com.redhat.rhevm.api.model.Event;
import com.redhat.rhevm.api.model.Host;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.model.User;
import com.redhat.rhevm.api.model.VM;
import com.redhat.rhevm.api.powershell.enums.PowerShellLogSeverity;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.powershell.util.UUID;

public class PowerShellEvent {

    public static List<Event> parse(PowerShellParser parser, String output) {
        List<Event> ret = new ArrayList<Event>();

        Map<String, XMLGregorianCalendar> dates = new HashMap<String, XMLGregorianCalendar>();
        String date = null;

        for (PowerShellParser.Entity entity : parser.parse(output)) {
            if (PowerShellParser.DATE_TYPE.equals(entity.getType())) {
                date = entity.getValue();
                continue;
            } else if (PowerShellParser.STRING_TYPE.equals(entity.getType())) {
                dates.put(date, PowerShellUtils.parseDate(entity.getValue()));
                date = null;
                continue;
            }

            ret.add(parseEvent(entity, dates));
        }

        return ret;
    }

    private static Event parseEvent(PowerShellParser.Entity entity, Map<String, XMLGregorianCalendar> dates) {
        Event event = new Event();

        event.setId(Long.toString(entity.get("id", Long.class)));
        event.setDescription(entity.get("message"));
        event.setCode(entity.get("logtype", Integer.class));
        event.setSeverity(entity.get("severity", PowerShellLogSeverity.class).map().value());

        if (dates.containsKey(entity.get("logtime"))) {
            event.setTime(dates.get(entity.get("logtime")));
        }

        if (entity.isSet("userid") && !entity.get("userid").equals(UUID.EMPTY)) {
            User user = new User();
            user.setId(entity.get("userid"));
            event.setUser(user);
        }
        if (entity.isSet("vmid") && !entity.get("vmid").equals(UUID.EMPTY)) {
            VM vm = new VM();
            vm.setId(entity.get("vmid"));
            event.setVm(vm);
        }
        if (entity.isSet("storagedomainid") && !entity.get("storagedomainid").equals(UUID.EMPTY)) {
            StorageDomain sd = new StorageDomain();
            sd.setId(entity.get("storagedomainid"));
            event.setStorageDomain(sd);
        }
        if (entity.isSet("hostid")) {
            Host host = new Host();
            host.setId(entity.get("hostid", Integer.class).toString());
            event.setHost(host);
        }
        if (entity.isSet("vmtemplateid") && !entity.get("vmtemplateid").equals(UUID.EMPTY)) {
            Template template = new Template();
            template.setId(entity.get("vmtemplateid"));
            event.setTemplate(template);
        }

        return event;
    }
}
