/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;

import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.model.BaseResource;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;

public abstract class AbstractPowerShellStorageDomainContentsResource<R extends BaseResource>
    extends AbstractPowerShellResource
    implements UriInfoProvider {

    private PowerShellStorageDomainResource parent;
    private UriInfo ui;

    public AbstractPowerShellStorageDomainContentsResource(PowerShellStorageDomainResource parent,
                                                           PowerShellPoolMap shellPools,
                                                           PowerShellParser parser) {
        super(shellPools, parser);
        this.parent = parent;
    }

    public UriInfo getUriInfo() {
        return ui;
    }

    @Context
    public void setUriInfo(UriInfo uriInfo) {
        ui = uriInfo;
    }

    public String getStorageDomainId() {
        return parent.getId();
    }

    protected String getDataCenterArg(StringBuilder buf) {
        buf.append("$datacenter = $null; ");
        buf.append("foreach ($dc in select-datacenter) { ");
        buf.append(  "$sds = @(); ");
        buf.append(  "try { ");
        buf.append(    "$sds = get-storagedomain -datacenterid $dc.datacenterid ");
        buf.append(  "} catch { } ");
        buf.append(  "foreach ($sd in $sds) { ");
        buf.append(    "if ($sd.storagedomainid -eq " + PowerShellUtils.escape(getStorageDomainId()) + ") { ");
        buf.append(      "$datacenter = $dc; ");
        buf.append(      "break ");
        buf.append(    "} ");
        buf.append(  "} ");
        buf.append(  "if ($datacenter -ne $null) { ");
        buf.append(    "break ");
        buf.append(  "} ");
        buf.append("} ");
        return "$datacenter.datacenterid";
    }

    public abstract R runAndParseSingle(String command);
    public abstract R addLinks(UriInfo uriInfo, R model);
}
