/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;
import java.util.concurrent.Executor;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.model.Role;
import com.redhat.rhevm.api.model.Roles;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.resource.RoleResource;
import com.redhat.rhevm.api.resource.AssignedRolesResource;

import static com.redhat.rhevm.api.common.util.CompletenessAssertor.validateParameters;
import static com.redhat.rhevm.api.powershell.resource.PowerShellUsersResource.getRoleArg;

public class PowerShellAssignedRolesResource extends UriProviderWrapper implements AssignedRolesResource {

    private String userId;

    public PowerShellAssignedRolesResource(String userId,
                                           Executor executor,
                                           PowerShellPoolMap shellPools,
                                           PowerShellParser parser,
                                           UriInfoProvider uriProvider) {
        super(executor, shellPools, parser, uriProvider);
        this.userId = userId;
    }

    public List<Role> runAndParse(String command) {
        return PowerShellRoleResource.runAndParse(getPool(), getParser(), userId, command);
    }

    public Role runAndParseSingle(String command) {
        return PowerShellRoleResource.runAndParseSingle(getPool(), getParser(), userId, command);
    }

    @Override
    public Roles list() {
        Roles ret = new Roles();
        StringBuilder getRoles = new StringBuilder();

        // roles assigned to a specific user
        getRoles.append("$u = get-user -userid ")
                .append(PowerShellUtils.escape(userId))
                .append(";$u.getroles()");
        for (Role Role : runAndParse(getRoles.toString())) {
            ret.getRoles().add(LinkHelper.addLinks(getUriInfo(), Role));
        }
        return ret;
    }

    @Override
    public Response add(Role role) {
        validateParameters(role, "name|id");
        StringBuilder buf = new StringBuilder();

        String roleArg = getRoleArg(role, buf);

        buf.append("attach-role -roleid ").append(roleArg).append(" -elementid ").append(userId);
        Role newRole = LinkHelper.addLinks(getUriInfo(), runAndParseSingle(buf.toString()));
        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(newRole.getId());

        return Response.created(uriBuilder.build()).entity(newRole).build();
    }

    @Override
    public void remove(String roleId) {
        StringBuilder detachRole = new StringBuilder();
        detachRole.append("detach-role -roleid ")
                  .append(PowerShellUtils.escape(roleId))
                  .append(" -elementid ").append(userId);
        PowerShellCmd.runCommand(getPool(), detachRole.toString());
    }

    @Override
    public RoleResource getRoleSubResource(String roleId) {
        return new PowerShellRoleResource(roleId, userId, executor, shellPools, parser, getUriProvider());
    }
}
