/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.concurrent.Executor;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.model.Action;
import com.redhat.rhevm.api.model.DataCenter;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.resource.AttachedStorageDomainResource;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;


public class PowerShellAttachedStorageDomainResource
    extends AbstractPowerShellActionableResource<StorageDomain>
    implements AttachedStorageDomainResource {

    private PowerShellAttachedStorageDomainsResource parent;

    PowerShellAttachedStorageDomainResource(PowerShellAttachedStorageDomainsResource parent,
                                            String storageDomainId,
                                            Executor executor,
                                            UriInfoProvider uriProvider,
                                            PowerShellPoolMap shellPools,
                                            PowerShellParser parser) {
        super(storageDomainId, executor, uriProvider, shellPools, parser);
        this.parent = parent;
    }

    public String getDataCenterId() {
        return parent.getDataCenterId();
    }

    protected StorageDomain addLinks(UriInfo uriInfo, StorageDomain storageDomain) {
        return parent.addLinks(uriInfo, storageDomain);
    }

    @Override
    protected StorageDomain getModel() {
        StorageDomain storageDomain = new StorageDomain();
        storageDomain.setId(getId());
        storageDomain.setDataCenter(new DataCenter());
        storageDomain.getDataCenter().setId(getDataCenterId());
        return storageDomain;
    }

    @Override
    public StorageDomain get() {
        StringBuilder buf = new StringBuilder();

        buf.append("get-storagedomain");
        buf.append(" -datacenterid " + PowerShellUtils.escape(getDataCenterId()));
        buf.append(" -storagedomainid " + PowerShellUtils.escape(getId()));

        return addLinks(getUriInfo(),
                        PowerShellStorageDomainResource.runAndParseSingle(getPool(), getParser(), buf.toString()));
    }

    @Override
    public Response activate(Action action) {
        return doAction(getUriInfo(), new AttachedStorageDomainActionTask(action, "activate-storagedomain"));
    }

    @Override
    public Response deactivate(Action action) {
        return doAction(getUriInfo(), new AttachedStorageDomainActionTask(action, "deactivate-storagedomain"));
    }

    private class AttachedStorageDomainActionTask extends AbstractPowerShellActionTask {
        public AttachedStorageDomainActionTask(Action action, String command) {
            super(action, command);
        }
        public void execute() {
            StringBuilder buf = new StringBuilder();

            buf.append(command);
            buf.append(" -datacenterid " + PowerShellUtils.escape(getDataCenterId()));
            buf.append(" -storagedomainid " + PowerShellUtils.escape(getId()));

            PowerShellCmd.runCommand(getPool(), buf.toString());
        }
    }
}
