/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.model.File;
import com.redhat.rhevm.api.model.Files;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.resource.FileResource;
import com.redhat.rhevm.api.resource.FilesResource;
import com.redhat.rhevm.api.powershell.model.PowerShellFile;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;

public class PowerShellFilesResource extends UriProviderWrapper implements FilesResource {

    private String storageDomainId;

    public PowerShellFilesResource(String storageDomainId,
                                  PowerShellPoolMap shellPools,
                                  PowerShellParser parser,
                                  UriInfoProvider uriProvider) {
        super(null, shellPools, parser, uriProvider);
        this.storageDomainId = storageDomainId;
    }

    @Override
    public Files list() {
        StringBuilder buf = new StringBuilder();
        buf.append("foreach ($d in get-datacenter -storagedomainid " + PowerShellUtils.escape(storageDomainId) + ") {");
        buf.append("if ($d.status -eq \"Up\") { ");
        buf.append("get-isoimages");
        buf.append(" -datacenterid $d.datacenterid;");
        buf.append("break;");
        buf.append("} }");
        Files ret = new Files();
        for (File file : PowerShellFile.parse(parser, PowerShellCmd.runCommand(getPool(), buf.toString()))) {
            ret.getFiles().add(addLinks(file, storageDomainId));
        }
        return ret;
    }

    @Override
    public FileResource getFileSubResource(String id) {
        return new PowerShellFileResource(id, storageDomainId, this);
    }

    public File addLinks(File file, String storageDomainId) {
        file.setStorageDomain(new StorageDomain());
        file.getStorageDomain().setId(storageDomainId);
        return LinkHelper.addLinks(getUriInfo(), file);
    }
}
