/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.model.NIC;
import com.redhat.rhevm.api.model.Nics;
import com.redhat.rhevm.api.model.NicInterface;
import com.redhat.rhevm.api.powershell.enums.PowerShellVmInterfaceType;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.resource.DevicesResource;

import static com.redhat.rhevm.api.common.util.CompletenessAssertor.validateParameters;


public class PowerShellNicsResource
    extends PowerShellReadOnlyNicsResource
    implements DevicesResource<NIC, Nics> {

    public PowerShellNicsResource(String parentId,
                                  PowerShellPoolMap shellPools,
                                  PowerShellParser parser,
                                  String getCommand,
                                  UriInfoProvider uriProvider) {
        super(parentId, shellPools, parser, getCommand, uriProvider);
    }

    @Override
    public Response add(NIC nic) {
        validateParameters(nic, "name", "network.id");
        StringBuilder buf = new StringBuilder();

        buf.append("$v = get-vm " + PowerShellUtils.escape(parentId) + ";");
        buf.append("foreach ($i in get-networks) {");
        buf.append("  if ($i.networkid -eq " + PowerShellUtils.escape(nic.getNetwork().getId()) + ") {");
        buf.append("    $n = $i");
        buf.append("  }");
        buf.append("}");

        buf.append("add-networkadapter");
        buf.append(" -vmobject $v");
        buf.append(" -interfacename " + PowerShellUtils.escape(nic.getName()));
        buf.append(" -networkname $n.name");
        if (nic.getInterface() != null) {
            NicInterface type = NicInterface.fromValue(nic.getInterface());
            if (type != null) {
                buf.append(" -interfacetype " + PowerShellVmInterfaceType.forModel(type).name());
            }
        }
        if (nic.getMac() != null && nic.getMac().getAddress() != null) {
            buf.append(" -macaddress " + PowerShellUtils.escape(nic.getMac().getAddress()));
        }

        nic = addLinks(runAndParseSingle(buf.toString()));

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(nic.getId());

        return Response.created(uriBuilder.build()).entity(nic).build();
    }

    @Override
    public void remove(String id) {
        StringBuilder buf = new StringBuilder();

        buf.append("$v = get-vm " + PowerShellUtils.escape(parentId) + ";");

        buf.append("foreach ($i in $v.GetNetworkAdapters()) {");
        buf.append("  if ($i.id -eq " + PowerShellUtils.escape(id) + ") {");
        buf.append("    $n = $i");
        buf.append("  }");
        buf.append("}");
        buf.append("remove-networkadapter -vmobject $v -networkadapterobject $n");

        PowerShellCmd.runCommand(getPool(), buf.toString());
    }

    @Override
    public PowerShellDeviceResource<NIC, Nics> getDeviceSubResource(String id) {
        return new PowerShellDeviceResource<NIC, Nics>(this, id);
    }

}
