/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;

import com.redhat.rhevm.api.model.BaseDevice;
import com.redhat.rhevm.api.model.BaseDevices;
import com.redhat.rhevm.api.resource.CreationResource;
import com.redhat.rhevm.api.resource.ReadOnlyDeviceResource;


public class PowerShellReadOnlyDeviceResource<D extends BaseDevice, C extends BaseDevices>
    implements ReadOnlyDeviceResource<D> {

    protected AbstractPowerShellDevicesResource<D, C> parent;
    protected String deviceId;

    public PowerShellReadOnlyDeviceResource(AbstractPowerShellDevicesResource<D, C> parent, String deviceId) {
        this.parent = parent;
        this.deviceId = deviceId;
    }

    @Override
    public D get() {
        List<D> devices = parent.getDevices();

        for (D d : devices) {
            if (deviceId.equals(d.getId())) {
                return parent.addLinks(d);
            }
        }

        return null;
    }

    @Override
    public CreationResource getCreationSubresource(String ids) {
        return new PowerShellCreationResource(ids,
                                              parent.getExecutor(),
                                              parent.getUriProvider(),
                                              parent.getPowerShellPoolMap(),
                                              parent.getParser());
    }
}
