/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;

import com.redhat.rhevm.api.model.NIC;
import com.redhat.rhevm.api.model.Nics;
import com.redhat.rhevm.api.model.Network;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.powershell.model.PowerShellNIC;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.resource.ReadOnlyDevicesResource;


public class PowerShellReadOnlyNicsResource extends AbstractPowerShellDevicesResource<NIC, Nics> implements ReadOnlyDevicesResource<NIC, Nics> {

    private String getCommand;

    public PowerShellReadOnlyNicsResource(String parentId,
                                          PowerShellPoolMap shellPools,
                                          PowerShellParser parser,
                                          String getCommand,
                                          UriInfoProvider uriProvider) {
        super(parentId, shellPools, parser, uriProvider);
        this.getCommand = getCommand;
    }

    public List<NIC> runAndParse(String command) {
        return PowerShellNIC.parse(getParser(), parentId, PowerShellCmd.runCommand(getPool(), command));
    }

    public NIC runAndParseSingle(String command) {
        List<NIC> nics = runAndParse(command);

        return !nics.isEmpty() ? nics.get(0) : null;
    }

    @Override
    public List<NIC> getDevices() {
        StringBuilder buf = new StringBuilder();

        buf.append("$v = " + getCommand + " " + PowerShellUtils.escape(parentId) + ";");
        buf.append("$v.GetNetworkAdapters()");

        return runAndParse(buf.toString());
    }

    /* Map the network names to network ID on the supplied network
     * interfaces. The powershell output only includes the network name.
     *
     * @param nic  the NIC to modify
     * @return     the modified NIC
     */
    private NIC lookupNetworkId(NIC nic) {
        StringBuilder buf = new StringBuilder();

        buf.append("$n = get-networks;");
        buf.append("foreach ($i in $n) {");
        buf.append("  if ($i.name -eq " + PowerShellUtils.escape(nic.getNetwork().getName()) + ") {");
        buf.append("    $i");
        buf.append("  }");
        buf.append("}");

        Network network = new Network();
        network.setId(PowerShellNetworkResource.runAndParseSingle(getPool(), getParser(), buf.toString()).getId());
        nic.setNetwork(network);

        return nic;
    }

    @Override
    public NIC addLinks(NIC nic) {
        return LinkHelper.addLinks(getUriInfo(), lookupNetworkId(nic));
    }

    @Override
    public Nics list() {
        Nics nics = new Nics();
        for (NIC nic : getDevices()) {
            nics.getNics().add(addLinks(nic));
        }
        return nics;
    }

    @Override
    public PowerShellReadOnlyDeviceResource<NIC, Nics> getDeviceSubResource(String id) {
        return new PowerShellReadOnlyDeviceResource<NIC, Nics>(this, id);
    }
}
