/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;

import javax.ws.rs.core.UriInfo;

import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.model.Templates;
import com.redhat.rhevm.api.powershell.model.PowerShellTemplate;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.resource.StorageDomainContentResource;
import com.redhat.rhevm.api.resource.StorageDomainContentsResource;

public class PowerShellStorageDomainTemplatesResource
    extends AbstractPowerShellStorageDomainContentsResource<Template>
    implements StorageDomainContentsResource<Templates, Template> {

    public PowerShellStorageDomainTemplatesResource(PowerShellStorageDomainResource parent,
                                              PowerShellPoolMap shellPools,
                                              PowerShellParser parser) {
        super(parent, shellPools, parser);
    }

    public List<PowerShellTemplate> runAndParse(String command) {
        return PowerShellTemplate.parse(getParser(), PowerShellCmd.runCommand(getPool(), command));
    }

    @Override
    public Template runAndParseSingle(String command) {
        List<PowerShellTemplate> templates = runAndParse(command);

        return !templates.isEmpty() ? templates.get(0) : null;
    }

    @Override
    public Template addLinks(UriInfo uriInfo, Template template) {
        template = JAXBHelper.clone("template", Template.class, template);

        template.setStorageDomain(new StorageDomain());
        template.getStorageDomain().setId(getStorageDomainId());

        return LinkHelper.addLinks(uriInfo, template);
    }

    @Override
    public Templates list() {
        StringBuilder buf = new StringBuilder();

        String dataCenterArg = getDataCenterArg(buf);

        buf.append("get-templateimportcandidates");
        buf.append(" -showall");
        buf.append(" -datacenterid " + dataCenterArg);
        buf.append(" -storagedomainid " + PowerShellUtils.escape(getStorageDomainId()));

        Templates ret = new Templates();
        for (PowerShellTemplate template : runAndParse(buf.toString())) {
            ret.getTemplates().add(addLinks(getUriInfo(), template));
        }
        return ret;
    }

    public StorageDomainContentResource<Template> getStorageDomainContentSubResource(String id) {
        return new PowerShellStorageDomainTemplateResource(this, id, executor, this, shellPools, getParser());
    }
}
